﻿using GeoAPI.IO;
using NetTopologySuite;
using NetTopologySuite.IO;
using System;
using ThinkGeo.MapSuite.Shapes;

namespace GettingStartedSample
{
    public class CustomFeature : Feature
    {
        public CustomFeature()
            : base()
        { }

        public CustomFeature(BaseShape baseShape)
           : base(baseShape)
        { }

        public CustomFeature(string wellKnownText)
          : base(wellKnownText)
        { }

        public CustomFeature(Feature sourceFeature)
          : base(sourceFeature.GetWellKnownBinary())
        { }

        public void WrapGeomotry()
        {
            var wkb = GetWellKnownBinary();
            var wkbReader = new WKBReader(NtsGeometryServices.Instance);
            var geometry = wkbReader.Read(wkb);
            var coordinates = geometry.Coordinates;
            for (int index = 1; index < coordinates.Length; index++)
            {
                var longitudeSpacing = coordinates[index].X - coordinates[index - 1].X;
                if (Math.Abs(longitudeSpacing) > 180)
                {
                    if (longitudeSpacing < 0)
                        coordinates[index].X += 360;
                    else
                        coordinates[index - 1].X += 360;
                }

                if (geometry.GeometryType.Equals("Polygon") && index == coordinates.Length - 1)
                {
                    longitudeSpacing = coordinates[index].X - coordinates[0].X;
                    if (Math.Abs(longitudeSpacing) > 180)
                    {
                        if (longitudeSpacing < 0)
                            coordinates[index].X += 360;
                        else
                            coordinates[0].X += 360;
                    }
                }
            }

            WKBWriter wkbWriter = new WKBWriter(ByteOrder.LittleEndian);
            byte[] result = wkbWriter.Write(geometry);
            SetWellKnownBinary(result);
        }

        public void UnwrapGeomotry()
        {
            var wkb = GetWellKnownBinary();
            var wkbReader = new WKBReader(NtsGeometryServices.Instance);
            var geometry = wkbReader.Read(wkb);
            var coordinates = geometry.Coordinates;
            foreach (var coordinate in coordinates)
                coordinate.X = coordinate.X > 180 ? coordinate.X - 360 : coordinate.X;

            WKBWriter wkbWriter = new WKBWriter(ByteOrder.LittleEndian);
            byte[] result = wkbWriter.Write(geometry);
            SetWellKnownBinary(result);
        }
    }
}
